/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1992 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 source code is provided to you solely for       */
/*    the purpose of assisting you in your development of OS/2 device        */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Developer Connection Device Driver       */
/*    Source Kit for OS/2. This Copyright statement may not be removed.      */
/*                                                                           */
/*****************************************************************************/
/*
*
*/
/************************** START OF SPECIFICATIONS ***************************/
/*                                                                            */
/*   SOURCE FILE NAME:  AUDSUBS.C                                             */
/*                                                                            */
/*   DESCRIPTIVE NAME: Audio device driver subroutines.                       */
/*                                                                            */
/*                                                                            */
/*   FUNCTION:                                                                */
/*              Provides routines to assist execution of the primary          */
/*              logic of the audio device driver (audiodd.c).                 */
/*              The routines in this sample are callable, but being stubs,    */
/*              don't do anything.  Still, the structure is useful.           */
/*                                                                            */
/*              Most of the routines equate directly to OS/2 2.0              */
/*              kernel <-> PDD interfaces.  Accordingly, the OS/2 2.0         */
/*              Physical Device Driver reference should be referenced for     */
/*              the routines that serve the purpose of kernel communication   */
/*                                                                            */                                          
/*              Other routines are specific to the Audio Device Driver.       */                                          
/*                                                                            */
/*                                                                            */
/*   NOTES:                                                                   */
/*      DEPENDENCIES: None                                                    */
/*      RESTRICTIONS: None                                                    */
/*                                                                            */
/* Change Log                                                                 */
/*                                                                            */
/*  Mark    yy/mm/dd  Programmer          Comment                             */
/*  ----    --------  ----------          -------                             */
/*          98/10/17  Vjacheslav Chibis   Original developer.                 */
/*       2000/01/26   Vjacheslav Chibis   USB audio input added               */
/* 05/05/2000 00/04/05 MB                 Added more complete init/cleanup    */
/*                                        code in Open/Close routines         */
/* 08/01/2000 00/08/01 MB                 Added detached devices check        */
/*                                                                            */
/**************************** END OF SPECIFICATIONS ***************************/


#define  INCL_DOSINFOSEG
#include "usbaud.h"
#include "audsubs.h"    // Function declarations

BOOL static CompBinaries( PULONG a, PULONG b, USHORT size );  

/******************* START OF SPECIFICATIONS **************************/
/*                                                                    */
/* SUBROUTINE NAME:  GetStreamTime                                    */
/*                                                                    */
/* DESCRIPTIVE NAME: GetStreamTime                                    */
/*                                                                    */
/* FUNCTION:    Get Stream Time                                       */
/*                                                                    */
/* CONTEXT: Task Time                                                 */
/*                                                                    */
/* ENTRY POINT :  GetStreamTime                                       */
/*    LINKAGE  :  CALL NEAR                                           */
/*                                                                    */
/* INPUT:  PSTREAM pStream  - a pointer to a stream                   */
/*                                                                    */
/* EXIT-NORMAL:  stream current time                                  */
/*                                                                    */
/* EXIT-ERROR:  N/A                                                   */
/*                                                                    */
/* EFFECTS:  NONE                                                     */
/*                                                                    */
/* INTERNAL REFERENCES:  NONE                                         */
/*                                                                    */
/* EXTERNAL REFERENCES:  NONE                                         */
/*                                                                    */
/******************* END  OF  SPECIFICATIONS **************************/
PULONG GetStreamTime (PSTREAM pStream) 
{
	return (&pStream->ulCumTime);
}

/******************* START OF SPECIFICATIONS **************************/
/*                                                                    */
/* SUBROUTINE NAME:  SetStreamTime                                    */
/*                                                                    */
/* DESCRIPTIVE NAME: SetStreamTime                                    */
/*                                                                    */
/* FUNCTION:    Set time for the stream specified                     */
/*                                                                    */
/* CONTEXT: Task Time                                                 */
/*                                                                    */
/* ENTRY POINT :  SetStreamTime                                       */
/*    LINKAGE  :  CALL NEAR                                           */
/*                                                                    */
/* INPUT:  PSTREAM pStream - ptr to stream                            */
/*         ULONG ulSetPos  - stream time in ms                        */
/*                                                                    */
/* EXIT-NORMAL:  always                                               */
/*                                                                    */
/* EXIT-ERROR:  N/A                                                   */
/*                                                                    */
/* EFFECTS:  pStream->ulCumTime                                       */
/*                                                                    */
/* INTERNAL REFERENCES:  NONE                                         */  
/*                                                                    */
/* EXTERNAL REFERENCES:                                               */
/*                                                                    */
/*    ROUTINES:  ReportEvent                                          */
/*                                                                    */
/******************* END  OF  SPECIFICATIONS **************************/

VOID SetStreamTime (PSTREAM pStream, ULONG ulSetPos)
{
	#ifdef DEBUG
	dsPrint1(DBG_SPECIFIC, "USBAUD: SetStreamTime at %ldms\r\n", ulSetPos);
	#endif
	pStream->ulCumTime = ulSetPos;
	ReportEvent(pStream, FALSE); // report event if bypassed
}
/******************* START OF SPECIFICATIONS **************************/
/*                                                                    */
/* SUBROUTINE NAME:  FindDevice                                       */
/*                                                                    */
/* DESCRIPTIVE NAME: Find device                                      */
/*                                                                    */
/* FUNCTION:    Returns attached device index                         */
/*                                                                    */
/* CONTEXT: Task Time                                                 */
/*                                                                    */
/* ENTRY POINT :  FindDevice                                          */
/*    LINKAGE  :  CALL NEAR                                           */
/*                                                                    */
/* INPUT:  UCHAR devAddr - searched device address                    */
/*         PUCHAR  pucIndex  - device index                           */
/*                                                                    */
/* EXIT-NORMAL:  0  - device found                                    */
/*                                                                    */
/* EXIT-ERROR:  ERROR                                                 */
/*                                                                    */
/* EFFECTS:  NONE                                                     */
/*                                                                    */
/* INTERNAL REFERENCES:  NONE                                         */  
/*                                                                    */
/*                                                                    */
/* EXTERNAL REFERENCES:  NONE                                         */
/*                                                                    */
/*                                                                    */
/******************* END  OF  SPECIFICATIONS **************************/

RC FindDevice(UCHAR devAddr, PUCHAR  pucIndex)
{
	for ( *pucIndex=0;*pucIndex<MAX_AUDIO_DEVICES;(*pucIndex)++ )
	{
		if ( gAudioDevices[*pucIndex].pDeviceInfo!=NULL )
		{
			if ( gAudioDevices[*pucIndex].pDeviceInfo->deviceAddress==devAddr )
				break;
		}
		if ( *pucIndex==MAX_AUDIO_DEVICES-1 )
			return (ERROR);
	}
	return (NO_ERROR);
}


/******************* START OF SPECIFICATIONS **************************/
/*                                                                    */
/* SUBROUTINE NAME:  DevOpen                                          */
/*                                                                    */
/* DESCRIPTIVE NAME: Open device                                      */
/*                                                                    */
/* FUNCTION:    The function of this routine is to process OS/2 IOCtl */
/*                       OPEN call                                    */
/*                                                                    */
/* CONTEXT: Task Time                                                 */
/*                                                                    */
/* ENTRY POINT :  DevOpen                                             */
/*    LINKAGE  :  CALL NEAR                                           */
/*                                                                    */
/* INPUT:  ULONG ulSysFileNum                                         */
/*                                                                    */
/* EXIT-NORMAL:  TRUE - device opened succesfully                     */
/*                                                                    */
/* EXIT-ERROR:  FALSE                                                 */
/*                                                                    */
/* EFFECTS:  NONE                                                     */
/*                                                                    */
/* INTERNAL REFERENCES:  NONE                                         */  
/*                                                                    */
/*                                                                    */
/* EXTERNAL REFERENCES:  NONE                                         */
/*                                                                    */
/*                                                                    */
/******************* END  OF  SPECIFICATIONS **************************/

USHORT DevOpen          (ULONG ulSysFileNum)
{
	register PSTREAM pStream;
	register USHORT streamIndex, bufferIndex, eventIndex;

	// RTMIDI uses this open and does not need
	// a stream object so he passes in a -1
	// sys_file_num
	#ifdef DEBUG
	dsPrint1(DBG_IRQFLOW, 
				"USBAUD: DevOpen entry ulSysFileNum=%lxh\r\n", 
				ulSysFileNum);
	#endif
	if ( ulSysFileNum != 0xFFFFFFFF )
	{
		//find a stream and init it
		pStream = GlobalTable.paStream;

		for ( streamIndex = 0; streamIndex < GlobalTable.usMaxNumStreams; streamIndex++ )
		{
			if ( (pStream->hStream == -1) && (pStream->ulSysFileNum == -1) )
				break;

			++pStream;
		} //end for

#ifdef DEBUG
dsPrint3(DBG_IRQFLOW, 
         "USBAUD: DevOpen entry pStream=%lxh (%d,%d)\r\n", 
         (ULONG)pStream, streamIndex, GlobalTable.usMaxNumStreams);
#endif

		if ( streamIndex == GlobalTable.usMaxNumStreams )
			//return (FALSE);
			return (TRUE);

//**************************************
// Found empty stream entry, so use it
// Fill Stream structure
//**************************************
		for ( bufferIndex=0; bufferIndex<MAXIOBUFFS; bufferIndex++ )
		{
			pStream->IOBuff[bufferIndex].lSize        =  0L;
			pStream->IOBuff[bufferIndex].pHead        =  NULL;
			pStream->IOBuff[bufferIndex].pTail        =  NULL;
			pStream->IOBuff[bufferIndex].lCount       =  0L;
			pStream->IOBuff[bufferIndex].ulPosition   =  0L;
			pStream->IOBuff[bufferIndex].usRunFlags   =  IOB_CHAIN_BUFFERS;
			pStream->IOBuff[bufferIndex].pBuffer      =  NULL;
			pStream->IOBuff[bufferIndex].bEmpty       =  TRUE;
		}
		pStream->ulSysFileNum        = ulSysFileNum;
		pStream->ucDeviceIndex       = INDEX_NOT_AVAILABLE;
		pStream->ulOperation         = 0L;
		pStream->ulFlags             = 0L;	  // clear flags
		pStream->ulCumTime           = 0L;
   	pStream->usTrackNum          = 0;
   	pStream->usCurrIOBuffIndex   = 0;
   	pStream->usFreeIOBuffIndex   = 0;
   	pStream->usLastIOBuffIndex   = 0;
   	pStream->ucRemained          = 0;
   	pStream->ucBuffers           = 0;
		pStream->usMode              = 0;
      pStream->bIsoOpened          = FALSE;

      pStream->ulVolume  = AUDIO_IGNORE;
      pStream->ulBass    = AUDIO_IGNORE;
      pStream->ulTreble  = AUDIO_IGNORE;


      {  // clear event queue
         EVENT_QUEUE *pEvent = pStream->pEventQueue;
      
         // search for the event
         for ( eventIndex=0; eventIndex<MAX_EVENT_QUEUE; eventIndex++, pEvent++ )
         {
            pEvent->hEvent = (HEVENT)(-1);
            pEvent->ulCuePoint = 0L;
         }
      }
	}

	return (TRUE);
}

/******************* START OF SPECIFICATIONS **************************/
/*                                                                    */
/* SUBROUTINE NAME:  DevClose                                         */
/*                                                                    */
/* DESCRIPTIVE NAME: Close evice                                      */
/*                                                                    */
/* FUNCTION:    The function of this routine is to process OS/2 IOCtl */
/*                       CLOSE call                                   */
/*                                                                    */
/* CONTEXT: Task Time                                                 */
/*                                                                    */
/* ENTRY POINT :  DevClose                                            */
/*    LINKAGE  :  CALL NEAR                                           */
/*                                                                    */
/* INPUT:  ULONG ulSysFileNum                                         */
/*                                                                    */
/* EXIT-NORMAL:  N/A                                                  */
/*                                                                    */
/* EXIT-ERROR:  N/A                                                   */
/*                                                                    */
/* EFFECTS:  NONE                                                     */
/*                                                                    */
/* INTERNAL REFERENCES:  NONE                                         */  
/*                                                                    */
/*                                                                    */
/* EXTERNAL REFERENCES:                                               */
/*            ROUTINES : GetFileStream                                */
/*                                                                    */
/*                                                                    */
/******************* END  OF  SPECIFICATIONS **************************/

VOID DevClose  (ULONG ulSysFileNum)
{
	register PSTREAM pStream=NULL ;
   USHORT           streamIndex;

	if ( ulSysFileNum != 0xFFFFFFFF )
	{
		// Clean up the stream
      for( streamIndex = 0; streamIndex < GlobalTable.usMaxNumStreams; streamIndex++ )
      {
   		pStream = GetFileStream(ulSysFileNum);
   		if ( pStream )
   		{
            ISOClose(pStream);   // 04/05/2000 MB
   			pStream->hStream = -1;
   			pStream->ulSysFileNum = -1;
            pStream->ucDeviceIndex = INDEX_NOT_AVAILABLE;
   			pStream->ulFlags = 0;		  // clear flags
#ifdef DEBUG
   dsPrint2(DBG_IRQFLOW, 
            "USBAUD: DevClose entry ulSysFileNum=%lxh, pStream=%lx\r\n", 
            ulSysFileNum, (ULONG)pStream);
#endif
   		}
         else
            break;
      }
	}	// if ulSysFileNum not FFFFFFFFF
}

/******************* START OF SPECIFICATIONS **************************/
/*                                                                    */
/* SUBROUTINE NAME:  DevChange                                        */
/*                                                                    */
/* DESCRIPTIVE NAME: Close evice                                      */
/*                                                                    */
/* FUNCTION:    Device specific operations to change device           */
/*                                operationality                      */
/*                                                                    */
/* CONTEXT: Task Time                                                 */
/*                                                                    */
/* ENTRY POINT :  DevChange                                           */
/*    LINKAGE  :  CALL NEAR                                           */
/*                                                                    */
/* INPUT:  LPMCI_AUDIO_CHANGE pRP_AC - ptr to                         */
/*                            Audio Change structure                  */
/*                                                                    */
/* EXIT-NORMAL:  N/A                                                  */
/*                                                                    */
/* EXIT-ERROR:  N/A                                                   */
/*                                                                    */
/* EFFECTS:  NONE                                                     */
/*                                                                    */
/* INTERNAL REFERENCES:  NONE                                         */  
/*                                                                    */
/*                                                                    */
/* EXTERNAL REFERENCES:  MAKEULONG()                                  */
/*                       MAKEUSHORT()                                 */
/*                       HIBYTE()                                     */
/*                       SetUnitControl()                             */
/*                       GetUnitControlValue()                        */
/*                                                                    */
/*                                                                    */
/******************* END  OF  SPECIFICATIONS **************************/

VOID DevChange  (PSTREAM pStream, LPMCI_AUDIO_CHANGE pRP_AC)			  // Change device type
{
	LPMCI_TRACK_INFO  pTrackInfo = (LPMCI_TRACK_INFO) pRP_AC->pvDevInfo;
	USHORT            usMasterVolume;
	PADEVICE          paDevice =  NULL;
	UCHAR             ucaIdx   =  INDEX_NOT_AVAILABLE;

#ifdef DEBUG
   dsPrint4(DBG_IRQFLOW, 
            "USBAUD: DevChange lIn=%lx, lOutpt=%lx, lMon=%lx, lVol=%lx\r\n",
            pRP_AC->lInput, pRP_AC->lOutput, pRP_AC->lMonitor, pRP_AC->lVolume);
   dsPrint4(DBG_IRQFLOW, 
            "USBAUD: DevChange lVolDly=%lx, lBlnce=%lx, lBDly=%lx, lTreble=%lx\r\n",
            pRP_AC->lVolumeDelay, pRP_AC->lBalance, pRP_AC->lBalanceDelay, pRP_AC->lTreble);
   dsPrint3(DBG_IRQFLOW, 
            "USBAUD: DevChange lBass=%lx, lPitch=%lx, lGain=%lx\r\n",
            pRP_AC->lBass, pRP_AC->lPitch, pRP_AC->lGain);
#endif

	if ( !gNoOfActiveAudioDevices || 
		  !pStream || ((ucaIdx=pStream->ucDeviceIndex)==INDEX_NOT_AVAILABLE) )
	{
		return;
	}

	paDevice=(PADEVICE)&gAudioDevices[ucaIdx];

	usMasterVolume =  ( pTrackInfo!= (LPMCI_TRACK_INFO) 0 ) ? pTrackInfo->usMasterVolume : 0; 

	#ifdef DEBUG
	dsPrint3(DBG_SPECIFIC, 
				"USBAUD: Vol=%lxh MVol=%lxh OutDevType=%xh\r\n",
				pRP_AC->lVolume,
				usMasterVolume,
				pRP_AC->rOutputList[0].ulDevType);
	#endif

	if ( pRP_AC->lBalance!=AUDIO_IGNORE )  // 04/05/2000 MB
   {
      pStream->ulBalance  =  pRP_AC->lBalance;
      gulBalance  =  pRP_AC->lBalance;
   }

   // check, if  we can set set the control for the specified endpoint
	if ( pRP_AC->lVolume!=AUDIO_IGNORE && 
		  ((pRP_AC->rOutputList[0].ulDevType != STEREO_LINE_OUTPUT) ||
			(pRP_AC->rOutputList[0].ulDevType != LEFT_LINE_OUTPUT)||
			(pRP_AC->rOutputList[0].ulDevType != RIGHT_LINE_OUTPUT)||
			(pRP_AC->rOutputList[0].ulDevType != SPEAKER_OUTPUT)) 
		)
	{
		register UCHAR ucChannel;      

//		pRP_AC->lVolume&=MAKEULONG(0, usMasterVolume);

		#ifdef DEBUG
		dsPrint1( DBG_SPECIFIC, 
					 "USBAUD: Vol=%lxh\r\n", 
					 pRP_AC->lVolume);
		#endif
		paDevice->ulBass     =  pRP_AC->lBass;
		paDevice->ulTreble   =  pRP_AC->lTreble;
		paDevice->ulVolume   =  pRP_AC->lVolume;
		paDevice->ulBalance  =  pRP_AC->lBalance;

      pStream->ulVolume = pRP_AC->lVolume;
      gulVolume = pRP_AC->lVolume;   // save as initial value

      for ( ucChannel=0; ucChannel<=paDevice->ucChannels;  ucChannel++ )
      {
         if ( paDevice->ulDevCh[ucChannel]&CHANNEL_FEATURE_VOLUME )
         {
            paDevice->usLeftVolume  = GetUnitControlValue(HIUSHORT(pRP_AC->lVolume),
                                                       paDevice->usMinVolume[ucChannel],    
                                                       paDevice->usMaxVolume[ucChannel]);


            paDevice->usRightVolume = paDevice->usLeftVolume;

            #ifdef DEBUG
            dsPrint4( DBG_SPECIFIC, 
                      "USBAUD : Set USB chan #%d Volume to %xh (min=%xh max=%xh)\r\n", 
                      ucChannel, 
                      paDevice->usLeftVolume, 
                      paDevice->usMinVolume[ucChannel], 
                      paDevice->usMaxVolume[ucChannel]);
            #endif
            // set volume
            SetUnitControl( ucaIdx, 
                            VOLUME_CONTROL, 
                            ucChannel, 
                            sizeof(paDevice->usLeftVolume), 
                            (PUCHAR)&paDevice->usLeftVolume
                          );

         }
      }

		if ( ( pRP_AC->lBass != AUDIO_IGNORE ) )
      {
         pStream->ulBass = pRP_AC->lBass;
         gulBass = pRP_AC->lBass;   // save as initial value
      

			for ( ucChannel=0; ucChannel<=paDevice->ucChannels;  ucChannel++ )
				if ( paDevice->ulDevCh[ucChannel]&CHANNEL_FEATURE_BASS )
				{
					paDevice->ucLeftBass = HIBYTE(GetUnitControlValue( HIUSHORT(pRP_AC->lBass),
																					MAKEUSHORT(0, paDevice->ucMinBass[ucChannel]),    
																					MAKEUSHORT(0, paDevice->ucMaxBass[ucChannel]))); 
				  #ifdef DEBUG
					dsPrint4(DBG_SPECIFIC, 
								"USBAUD : Set USB ch=%d Bass to %xh (min=%xh max=%xh)\r\n",
								ucChannel, 
								paDevice->ucLeftBass, paDevice->ucMinBass[ucChannel], 
								paDevice->ucMaxBass[ucChannel]);
				  #endif
					// set bass
					SetUnitControl( ucaIdx, 
										 BASS_CONTROL, 
										 ucChannel, 
										 sizeof(paDevice->ucLeftBass), 
										 (PUCHAR)&paDevice->ucLeftBass
									  );
				}
      }
		if ( ( pRP_AC->lTreble != AUDIO_IGNORE ) )
      {
         pStream->ulTreble = pRP_AC->lTreble;
         gulTreble = pRP_AC->lTreble;   // save as initial value
         
			for ( ucChannel=0; ucChannel<=paDevice->ucChannels;  ucChannel++ )
			{
				if ( paDevice->ulDevCh[ucChannel]&CHANNEL_FEATURE_TREBLE )
				{
					paDevice->ucLeftTreble  =  HIBYTE(GetUnitControlValue(HIUSHORT(pRP_AC->lTreble),
																						MAKEUSHORT(0, paDevice->ucMinTreble[ucChannel]),    
																						MAKEUSHORT(0, paDevice->ucMaxTreble[ucChannel])));
//					gOLDAC.lTreble = pRP_AC->lTreble;

					#ifdef DEBUG
					dsPrint4( DBG_SPECIFIC, 
								 "USBAUD : Set USB ch=%d treble to %xh (min=%xh max=%xh\r\n",
								 ucChannel, 
								 paDevice->ucLeftTreble, paDevice->ucMinTreble[ucChannel], 
								 paDevice->ucMaxTreble[ucChannel]);
					#endif
					// set treble
					SetUnitControl( ucaIdx, 
										 TREBLE_CONTROL, 
										 ucChannel, 
										 sizeof(paDevice->ucLeftTreble), 
										 (PUCHAR)&paDevice->ucLeftTreble
									  );
				} // if 
			}// for
      }
	}
	else
	{
		#ifdef DEBUG
		dsPrint3(DBG_SPECIFIC, 
					"USBAUD: Not setting Unit controls vol=%lxh, Master=%lxh devtype=%xh\r\n",
					pRP_AC->lVolume, 
					usMasterVolume, 
					pRP_AC->rOutputList[0].ulDevType);
		#endif
	}

	#ifdef DEBUG
	dsPrint2(DBG_SPECIFIC, "USBAUD: Bass=%xh Treble=%xh\r\n",  
				pRP_AC->lBass, 
				pRP_AC->lTreble);
	#endif

}


/******************* START OF SPECIFICATIONS **************************/
/*                                                                    */
/* SUBROUTINE NAME:  EnqueueEvent                                     */
/*                                                                    */
/* DESCRIPTIVE NAME: Enqueue Event                                    */
/*                                                                    */
/* FUNCTION:    Generates detach procedure for specified device       */
/*              When processing data, the PDD watches for             */
/*              specific events.  For example, when data at           */
/*              certain stream position is processed, the             */
/*              PDD tells the stream handler of the encountered       */
/*              event (eg setpositionadvise on every ...)             */
/*             The PDD has to have a mechanish of noticing            */
/*             these events.                                          */
/*                                                                    */
/* CONTEXT: Task Time                                                 */
/*                                                                    */
/* ENTRY POINT :  EnqueueEvent                                        */
/*    LINKAGE  :  CALL NEAR                                           */
/*                                                                    */
/* INPUT:  HEVENT hEvent  - event handler                             */
/*         ULONG ulCuePoint - event time in ms                        */
/*         PSTREAM pStream  - a pointer to a stream                   */
/*                                                                    */
/*                                                                    */
/* EXIT-NORMAL:  NO_ERROR                                             */
/*                                                                    */
/* EXIT-ERROR:  ERROR_TOO_MANY_EVENTS                                 */
/*                                                                    */
/* EFFECTS:  NONE                                                     */
/*                                                                    */
/* INTERNAL REFERENCES:                                               */  
/*                                                                    */
/*    ROUTINES:  NONE                                                 */
/*                                                                    */
/* EXTERNAL REFERENCES:                                               */
/*                                                                    */
/*    ROUTINES:  NONE                                                 */
/*                                                                    */
/******************* END  OF  SPECIFICATIONS **************************/

ULONG EnqueueEvent (HEVENT hEvent, ULONG ulCuePoint, PSTREAM pStream)
{
	register  USHORT  i;
	EVENT_QUEUE       *pEvent;
	BOOL              bNodeAvail = FALSE;


	// find an empty node in the queue
	for ( i = 0, pEvent = pStream->pEventQueue; i < MAX_EVENT_QUEUE; i++,pEvent++ )
	{
		if ( pEvent->hEvent == (HEVENT)(-1) )
		{
			bNodeAvail = TRUE;
			break;
		}
	}

	// if no node available, return error
	if ( !bNodeAvail )
	{
		return (ERROR_TOO_MANY_EVENTS);
	}


	// set the event queue node
	pEvent->hEvent = hEvent;
	pEvent->ulCuePoint = ulCuePoint;


	#ifdef DEBUG
	dsPrint1( DBG_SPECIFIC, 
				 "USBAUD: EnqueueEvent : ulCuePoint=%lxh\r\n", 
				 ulCuePoint);
	#endif
	return (NO_ERROR);
}


/******************* START OF SPECIFICATIONS **************************/
/*                                                                    */
/* SUBROUTINE NAME:  DequeueEvent                                     */
/*                                                                    */
/* DESCRIPTIVE NAME: Dequeue Event                                    */
/*                                                                    */
/* FUNCTION:                                                          */
/*                                                                    */
/*                                                                    */
/* CONTEXT: Task Time                                                 */
/*                                                                    */
/* ENTRY POINT :  DequeueEvent                                        */
/*    LINKAGE  :  CALL NEAR                                           */
/*                                                                    */
/* INPUT:  HEVENT hEvent - event Handler ID                           */
/*         PSTREAM pStream - a pointer to a stream                    */
/*                                                                    */
/*                                                                    */
/* EXIT-NORMAL:  NO_ERROR                                             */
/*                                                                    */
/* EXIT-ERROR:  ERROR_TOO_MANY_EVENTS                                 */
/*                                                                    */
/* EFFECTS:  NONE                                                     */
/*                                                                    */
/* INTERNAL REFERENCES:                                               */  
/*                                                                    */
/*    ROUTINES:  NONE                                                 */
/*                                                                    */
/* EXTERNAL REFERENCES:                                               */
/*                                                                    */
/*    ROUTINES:  NONE                                                 */
/*                                                                    */
/******************* END  OF  SPECIFICATIONS **************************/

ULONG DequeueEvent (HEVENT hEvent, PSTREAM pStream)
{
	register USHORT i = 0;
	EVENT_QUEUE *pEvent = pStream->pEventQueue;

	// search for the event
	for ( ; i<MAX_EVENT_QUEUE; i++, pEvent++ )
	{
		if ( pEvent->hEvent == hEvent )
		{
			pEvent->hEvent = (HEVENT)(-1); // clean up the node //
			pEvent->ulCuePoint = 0L;
			break;
		}
	}
	return (NO_ERROR);
}

/******************* START OF SPECIFICATIONS **************************/
/*                                                                    */
/* SUBROUTINE NAME:  GetFileStream                                    */
/*                                                                    */
/* DESCRIPTIVE NAME: Get File Stream                                  */
/*                                                                    */
/* FUNCTION:  The function of this routine is to obtain stream        */
/*             for specified system file #                            */
/*                                                                    */
/*                                                                    */
/* CONTEXT: Task Time                                                 */
/*                                                                    */
/* ENTRY POINT :  GetFileStream                                       */
/*    LINKAGE  :  CALL NEAR                                           */
/*                                                                    */
/* INPUT:  ULONG ulSysFileNum                                         */
/*                                                                    */
/*                                                                    */
/* EXIT-NORMAL:  pointer to stream data structure                     */
/*                                                                    */
/* EXIT-ERROR:  NULL                                                  */
/*                                                                    */
/* EFFECTS:  NONE                                                     */
/*                                                                    */
/* INTERNAL REFERENCES:                                               */  
/*                                                                    */
/*    ROUTINES:  NONE                                                 */
/*                                                                    */
/* EXTERNAL REFERENCES:                                               */
/*                                                                    */
/*    ROUTINES:  NONE                                                 */
/*                                                                    */
/******************* END  OF  SPECIFICATIONS **************************/

PSTREAM GetFileStream (ULONG ulSysFileNum )
{
	PSTREAM pStream = GlobalTable.paStream; // get pointer to active stream
	register USHORT i = 0;

	//search for the stream
	for ( ; i < GlobalTable.usMaxNumStreams; i++ )
	{
		if ( pStream->ulSysFileNum == ulSysFileNum )
			return ( pStream );
		++pStream;
	}
	return ( NULL );
}	// GetFileStream //


/********************** START OF SPECIFICATIONS ***********************/
/*                                                                    */
/* SUBROUTINE NAME:  GetBestFit                                       */
/*                                                                    */
/* DESCRIPTIVE NAME:  Get best fit for audio data format              */
/*                                                                    */
/* FUNCTION:  The function of this routine is find interface          */
/*            that gives bst fit for required audio format.           */
/*                                                                    */
/* NOTES:                                                             */
/*                                                                    */
/* CONTEXT: Task time                                                 */
/*                                                                    */
/* ENTRY POINT:  GetBestFit                                           */
/*    LINKAGE:  CALL NEAR                                             */
/*                                                                    */
/* INPUT:  PUCHAR     configurationData - far pointer to device       */
/*                      configuration data buffer                     */
/*         UCHAR bNumConfigurations - number of configurations for    */
/*                      device                                        */
/*         UCHAR configurationValue - current configuration           */
/*         UCHAR interface - interface                                */
/*         PULONG     dataType - required data type value, changed    */
/*                           to the best fit if does not match        */
/*         PULONG        samplingRate - required sampling rate,       */
/*                           changed to the best fit if does not match*/
/*         PULONG        channels - required no of channels, changed  */
/*                           to the best fit if does not match        */
/*         PULONG        bitsPerSample- required bits per sample,     */
/*                          changed to the best fit if does not match */
/*         PUCHAR        altInterface - set to the best fit interface */
/*                          number
/*                                                                    */
/* EXIT-NORMAL: TRUE if format descriptor unit found                  */
/*                                                                    */
/* EXIT-ERROR: FALSE if failed to locate format descriptor            */
/*                                                                    */
/* EFFECTS: none                                                      */
/*                                                                    */
/* INTERNAL REFERENCES:  GetNextDescriptor                            */
/*                                                                    */
/* EXTERNAL REFERENCES:  None                                         */
/*                                                                    */
/************************ END OF SPECIFICATIONS ***********************/

// Format type codes
#define  FORMAT_TYPE_UNDEFINED         0x00
#define  FORMAT_TYPE_I                 0x01
#define  FORMAT_TYPE_II                0x02
#define  FORMAT_TYPE_III               0x03

// Audio Data Format Type I Codes
#define  TYPE_I_UNDEFINED              0x0000
#define  TYPE_I_PCM                    0x0001
#define  TYPE_I_PCM8                   0x0002
#define  TYPE_I_IEEE_FLOAT             0x0003
#define  TYPE_I_ALAW                   0x0004
#define  TYPE_I_MULAW                  0x0005

// Audio Data Format Type II Codes
#define  TYPE_II_UNDEFINED             0x1000
#define  TYPE_II_MPEG                  0x1001
#define  TYPE_II_AC3                   0x1002

// Audio Data Format Type II Codes
#define  TYPE_III_UNDEFINED            0x2000
#define  TYPE_III_IEC1937_AC3          0x2001
#define  TYPE_III_IEC1937_MPEG1_L1     0x2002
#define  TYPE_III_IEC1937_MPEG1_L23    0x2003
#define  TYPE_III_IEC1937_MPEG2_NOEXT  0x2003
#define  TYPE_III_IEC1937_MPEG2_EXT    0x2004
#define  TYPE_III_IEC1937_MPEG2_L1_LS  0x2005
#define  TYPE_III_IEC1937_MPEG2_L23_LS 0x2006


#pragma optimize ("",off)
BOOL GetBestFit( PUCHAR       configurationData,
					  UCHAR        bNumConfigurations,
					  UCHAR        bConfigurationValue,
					  UCHAR        interface,
					  PULONG       dataType,
					  PULONG       samplingRate,
					  PULONG       channels,
					  PULONG       bitsPerSample,
					  PUCHAR       altInterface
					)
{
	DeviceConfiguration FAR *devConf;
	UCHAR                   configIndex;
	PUCHAR currBytePtr, lastBytePtr;
	DeviceDescHead FAR      *descHead;
	DeviceInterface FAR     *interfaceDesc;
	Type1FDescriptor FAR    *formatDescriptor=NULL;
	ASIntfDescriptor FAR    *asIntfDescriptor;
	BOOL                    interfaceFound=FALSE, srcCode=FALSE;
	UCHAR                   curAltInterface, bestAltInterface=0;
	USHORT                  wFormatTag, reqFormatTag;
	ULONG                   minVal[4]={0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff};
	ULONG                   curVal[4];
	ULONG                   samplingFreq, diff, bestRate;
	UCHAR                   samplIndx;
	UCHAR                   bestFitChan=0, bestFitBits=0;
	ULONG                   bestFitRate=0;
	ULONG                   bestFitType=0;

	// convert MMPM/2 data type value to USB value
	switch ( *dataType )
	{
		case DATATYPE_WAVEFORM:
			if ( *bitsPerSample==8 )
				reqFormatTag=TYPE_I_PCM8;
			else
				reqFormatTag=TYPE_I_PCM;
			break;
		case DATATYPE_MULAW:
			reqFormatTag=TYPE_I_MULAW;
			break;
		case DATATYPE_ALAW:
			reqFormatTag=TYPE_I_ALAW;
			break;
		case DATATYPE_MPEG1AUDIO:
			reqFormatTag=TYPE_III_IEC1937_MPEG1_L1;
			break;
		default:
			reqFormatTag=0;
			break;
	}

	devConf=(DeviceConfiguration FAR *)configurationData;
	for ( configIndex=0; configIndex<bNumConfigurations;
		 configIndex++ )
	{
		currBytePtr=(PUCHAR)devConf;
		lastBytePtr=currBytePtr+devConf->wTotalLength;
		descHead=(DeviceDescHead FAR *)(currBytePtr+devConf->bLength);
		if ( devConf->bConfigurationValue==bConfigurationValue )
		{	// process required configuration descriptors
			for ( descHead=(DeviceDescHead FAR *)(currBytePtr+devConf->bLength); descHead;
				 descHead=GetNextDescriptor( descHead, lastBytePtr) )
			{
				#ifdef DEBUG
				dsPrint2( DBG_DETAILED, 
							 "GetBestFit : DESC Type=%d length=%d\r\n", 
							 descHead->bDescriptorType, 
							 descHead->bLength);
				#endif
				switch ( descHead->bDescriptorType )
				{
					case DESC_INTERFACE:
						interfaceDesc=(DeviceInterface FAR *)descHead;
						interfaceFound= interfaceDesc->bInterfaceNumber==interface;	// TRUE if this is required interface
						curAltInterface=interfaceDesc->bAlternateSetting;	// save current alternate setting value
						wFormatTag=0;	// reset format tag
						break;
					case CS_INTERFACE:
					#ifdef DEBUG
						dsPrint2(DBG_DETAILED, 
									"CS_INTERFACE subtype=%d found=%d\r\n", 
									descHead->bDescriptorSubtype, interfaceFound);
					#endif
						if ( !interfaceFound ) // bypass processing if descriptor does not belong to required interface
							break;
						switch ( descHead->bDescriptorSubtype )
						{
							case AS_GENERAL:
								asIntfDescriptor=(ASIntfDescriptor FAR *)descHead;
								wFormatTag=asIntfDescriptor->wFormatTag;	// save interface audio data type value
								break;
							case FORMAT_TYPE:
								srcCode=TRUE;	// format descriptor found
								formatDescriptor=(Type1FDescriptor FAR *)descHead;

								// calculate data type discrepancy
								if ( reqFormatTag==wFormatTag )
									curVal[0]=0;
								else
									curVal[0]=wFormatTag;
								#ifdef DEBUG

								dsPrint4(DBG_DETAILED, "FORMAT_TYPE_I len=%d type=%d subtype=%d formattype=%d\r\n",
											formatDescriptor->bLength,
											formatDescriptor->bDescriptorType,
											formatDescriptor->bDescriptorSubtype,
											formatDescriptor->bFormatType);
									#endif


								switch ( formatDescriptor->bFormatType )
								{
									case FORMAT_TYPE_I:

											#ifdef DEBUG

										dsPrint3(DBG_DETAILED, "FORMAT_TYPE_I ch=%d frame=%d bit=%d\r\n",
													formatDescriptor->bNrChannels,
													formatDescriptor->bSubFrameSize, 
													formatDescriptor->bBitResolution);

									#endif

										formatDescriptor=(Type1FDescriptor FAR *)descHead;
										// calculate current sampling rate, channel no, bits per sample discrepancies

										// process number of channels, prefer settings with higher no of channels
										if ( *channels<=(ULONG)formatDescriptor->bNrChannels )
											curVal[1]=formatDescriptor->bNrChannels-*channels;
										else
											curVal[1]=(*channels-formatDescriptor->bNrChannels)|0x0000ff00;

										// process bits per sample
										if ( formatDescriptor->bBitResolution==8 && curVal[0] )
										{
											if ( *bitsPerSample<=(ULONG)formatDescriptor->bBitResolution )
												curVal[2]=formatDescriptor->bBitResolution-*bitsPerSample|0x0000ff00;
											else
												curVal[2]=*bitsPerSample-formatDescriptor->bBitResolution;
										}
										else
										{
											if ( *bitsPerSample<(ULONG)formatDescriptor->bBitResolution )
												curVal[2]=formatDescriptor->bBitResolution-*bitsPerSample;
											else
												curVal[2]=*bitsPerSample-formatDescriptor->bBitResolution;
										}

										// process sampling frequencies
										if ( formatDescriptor->bSamFreqType )
										{	// discrete number of sampling frequencies
											curVal[3]=0xffffffff; bestRate=0;
											for ( samplIndx=0; samplIndx<formatDescriptor->bSamFreqType; samplIndx++ )
											{
												samplingFreq=*((PULONG)(&formatDescriptor->tSamFreq[samplIndx][0]));
												samplingFreq&=0x00ffffff;
												#ifdef DEBUG
												dsPrint3(DBG_DETAILED, "idx=%d freq=%ld samplingRate=%ld\r\n", 
															samplIndx, 
															samplingFreq, 
															*samplingRate);
												#endif

												diff=samplingFreq>*samplingRate ? samplingFreq-*samplingRate : *samplingRate-samplingFreq;
												if ( diff<curVal[3] )
												{
													curVal[3]=diff;
													bestRate=samplingFreq;
												}
											}
										}
										else
										{	// continuos sampling range
											curVal[3]=0xffffffff; bestRate=0;
											samplingFreq=*((PULONG)(&formatDescriptor->tSamFreq[0][0]));
											samplingFreq&=0x00ffffff;
											if ( samplingFreq>*samplingRate )
											{
												curVal[3]=samplingFreq-*samplingRate;
												bestRate=samplingFreq;
											}
											else
											{
												samplingFreq=*((ULONG  FAR *)(&formatDescriptor->tSamFreq[0][1]));
												samplingFreq&=0x00ffffff;
												if ( samplingFreq<*samplingRate )
												{
													curVal[3]=*samplingRate-samplingFreq;
													bestRate=samplingFreq;
												}
												else
												{
													curVal[3]=0;
													bestRate=*samplingRate;
												}
											}
										}

										#ifdef DEBUG
										dsPrint4(DBG_DETAILED, "GetBestFit 1 : alt=%d Rate=%ld chan=%d bits=%d\r\n", 
													curAltInterface,
													bestRate,
													formatDescriptor->bNrChannels,
													formatDescriptor->bBitResolution);
											#endif


										if ( CompBinaries( minVal, curVal, sizeof(minVal)/sizeof(ULONG) ) )
										{	// save values if current interface is closer to requirements than previous
											movmem((PSZ)&minVal, (PSZ)&curVal, sizeof(minVal));

											bestFitChan = formatDescriptor->bNrChannels;
											bestFitBits = formatDescriptor->bBitResolution;
											bestFitRate = bestRate;
											bestFitType = wFormatTag;
											bestAltInterface = curAltInterface;



											if ( bestFitChan==(UCHAR)*channels && 
												  bestFitRate==*samplingRate && 
												  bestFitBits==(UCHAR)*bitsPerSample )
											{
												#ifdef DEBUG
												dsPrint(DBG_DETAILED, "GetBestFit : Exact match!\r\n");
												#endif
												goto exact;
											}


											#ifdef DEBUG
											dsPrint4(DBG_DETAILED, "GetBestFit 2 : alt=%d Rate=%ld chan=%d bits=%d\r\n", 
														bestAltInterface,
														bestFitRate,
														bestFitChan,
														bestFitBits);
											dsPrint4(DBG_DETAILED, "mivVal = %lxh %lxh %lxh %lxh\r\n", 
														minVal[0], minVal[1], minVal[2], minVal[3]);
											#endif
											break;
										}
										break;

									case FORMAT_TYPE_II:
									#ifdef DEBUG
										dsPrint(DBG_DETAILED, "GetBestFit : FORMAT_TYPE_II\r\n");
									#endif
										break;

									case FORMAT_TYPE_III:
									#ifdef DEBUG
										dsPrint(DBG_DETAILED, "GetBestFit : FORMAT_TYPE_III\r\n");
									#endif

										break;
									default:
										break;
								}
								break;

							default:
								break;
						}
						break;
					default:
						break;
				}
			}
			break;
		}
		devConf=(DeviceConfiguration FAR *)lastBytePtr;	// point to next configuration block
	}
	exact:

	*altInterface = bestAltInterface;
	*samplingRate = bestFitRate;
	*channels     = (ULONG)bestFitChan;
	*bitsPerSample = (ULONG)bestFitBits;

	#ifdef DEBUG
	dsPrint4( DBG_DETAILED, "GetBestFit:FINAL: alt=%d Rate=%ld chan=%d bits=%d\r\n", 
				 *altInterface,
				 *samplingRate,
				 *channels,
				 *bitsPerSample);
	#endif

	// convert USB data type value to MMPM/2 value
	switch ( bestFitType )
	{
		case TYPE_I_PCM8:
		case TYPE_I_PCM:
			*dataType=DATATYPE_WAVEFORM;
			break;
		case TYPE_I_MULAW:
			*dataType=DATATYPE_MULAW;
			break;
		case TYPE_I_ALAW:
			*dataType=DATATYPE_ALAW;
			break;
		case TYPE_III_IEC1937_MPEG1_L1:
			*dataType=DATATYPE_MPEG1AUDIO;
			break;
		default:
			*dataType=DATATYPE_NULL;
			break;
	}

	return (srcCode);
}

/********************** START OF SPECIFICATIONS ***********************/
/*                                                                    */
/* SUBROUTINE NAME:  CompBinaries                                     */
/*                                                                    */
/* DESCRIPTIVE NAME:  Compare two binary values                       */
/*                                                                    */
/* FUNCTION:  The function of this routine is to compare binary       */
/*            values stored in 2 arrays.                              */
/*                                                                    */
/* NOTES:                                                             */
/*                                                                    */
/* CONTEXT: Task time                                                 */
/*                                                                    */
/* ENTRY POINT:  CompBinaries                                         */
/*    LINKAGE:  CALL NEAR                                             */
/*                                                                    */
/* INPUT:  PULONG     a - far pointer to the first ULONG array        */
/*                      storing binary value                          */
/*         PULONG     b - far pointer to the second ULONG array       */
/*                      storing binary value                          */
/*         USHORT size - no of elements in arrays                     */
/*                                                                    */
/* EXIT-NORMAL: TRUE if a>b                                           */
/*              FALSE a<=b                                            */
/*                                                                    */
/* EXIT-ERROR: none                                                   */
/*                                                                    */
/* EFFECTS: none                                                      */
/*                                                                    */
/* INTERNAL REFERENCES:  none                                         */
/*                                                                    */
/* EXTERNAL REFERENCES:  None                                         */
/*                                                                    */
/************************ END OF SPECIFICATIONS ***********************/
#pragma optimize ("",on)
BOOL static CompBinaries( PULONG a, PULONG b, USHORT size )
{
	USHORT   index;

	for ( index=0; index<size; index++ )
	{
		#ifdef DEBUG
		dsPrint3( DBG_DETAILED, 
					 "CompBinaries a=%lxh b=%lxh idx=%xh\r\n", 
					 a[index], 
					 b[index], 
					 index);
		#endif


		if ( a[index]>b[index] )
		{
			return (TRUE);
		}
		else if ( a[index]<b[index] )
			break;
	}

	return (FALSE);
}


/********************** START OF SPECIFICATIONS ***********************/
/*                                                                    */
/* SUBROUTINE NAME:  CheckChannelFeature                              */
/*                                                                    */
/* DESCRIPTIVE NAME:  Check channel feature                           */
/*                                                                    */
/* FUNCTION:  The function of this routine is to check audio channel  */
/*            capability to support specific feature.                 */
/*                                                                    */
/* NOTES:                                                             */
/*                                                                    */
/* CONTEXT: Task time                                                 */
/*                                                                    */
/* ENTRY POINT:  CheckChannelFeature                                  */
/*    LINKAGE:  CALL NEAR                                             */
/*                                                                    */
/* INPUT:  FUnitDescriptor FAR *featureDesc - far pointer to feature  */
/*                      unit descriptor                               */
/*         USHORT feature - feature to check                          */
/*                      device                                        */
/*         UCHAR channel - channel number to check (0 is used for     */
/*                      master channel)                               */
/*                                                                    */
/* EXIT-NORMAL: TRUE if feature supported by specified channel        */
/*              FALSE if feature is not supported by specified channel*/
/*                                                                    */
/* EXIT-ERROR: FALSE if       channel no specified                    */
/*                                                                    */
/* EFFECTS: none                                                      */
/*                                                                    */
/* INTERNAL REFERENCES:  none                                         */
/*                                                                    */
/* EXTERNAL REFERENCES:  None                                         */
/*                                                                    */
/************************ END OF SPECIFICATIONS ***********************/
BOOL CheckChannelFeature( FUnitDescriptor FAR *featureDesc, USHORT feature, UCHAR channel)
{
	PUSHORT     chanFeature;
	PUCHAR      featureBMAddr;
	PUCHAR      lastDescByte;


	if ( !featureDesc )
	{
		return (FALSE);
	}

	featureBMAddr = ((PUCHAR)&featureDesc->bmaControls)+channel*featureDesc->bControlSize;
	lastDescByte  = ((PUCHAR)featureDesc)+featureDesc->bLength;

	if ( lastDescByte<featureBMAddr+featureDesc->bControlSize )
	{
		return (FALSE);
	}


	chanFeature = (PUSHORT)featureBMAddr;  

	return (((*chanFeature)&feature)!=0);
}

/********************** START OF SPECIFICATIONS ***********************/
/*                                                                    */
/* SUBROUTINE NAME:  GetChannelCount                                  */
/*                                                                    */
/* DESCRIPTIVE NAME:  Get Channel Count                               */
/*                                                                    */
/* FUNCTION:  The function of this routine is to retrieve number of   */
/*            logical channels described in feature unit descriptor.  */
/*                                                                    */
/* NOTES:                                                             */
/*                                                                    */
/* CONTEXT: Task time                                                 */
/*                                                                    */
/* ENTRY POINT:  GetChannelCount                                      */
/*    LINKAGE:  CALL NEAR                                             */
/*                                                                    */
/* INPUT:  FUnitDescriptor FAR *featureDesc - far pointer to feature  */
/*                      unit descriptor                               */
/*                                                                    */
/* EXIT-NORMAL: no of logical channels described in feature unit      */
/*              (this does not include master channel)                */
/*                                                                    */
/* EXIT-ERROR: none                                                   */
/*                                                                    */
/* EFFECTS: none                                                      */
/*                                                                    */
/* INTERNAL REFERENCES:  none                                         */
/*                                                                    */
/* EXTERNAL REFERENCES:  None                                         */
/*                                                                    */
/************************ END OF SPECIFICATIONS ***********************/
UCHAR GetChannelCount( FUnitDescriptor FAR *featureDesc )
{
	return (featureDesc?(UCHAR )((featureDesc->bLength-7)/featureDesc->bControlSize-1):(UCHAR)0);
}

/********************** START OF SPECIFICATIONS ***********************/
/*                                                                    */
/* SUBROUTINE NAME:  SetChannelsCapabilities                          */
/*                                                                    */
/* DESCRIPTIVE NAME:  Set Channels Capabilities                       */
/*                                                                    */
/* FUNCTION:  The function of this routine is to                      */
/*                            Set Channels Capabilities               */
/*                                                                    */
/* NOTES:                                                             */
/*                                                                    */
/* CONTEXT: Task time                                                 */
/*                                                                    */
/* ENTRY POINT:  GetChannelCount                                      */
/*    LINKAGE:  CALL NEAR                                             */
/*                                                                    */
/* INPUT:  FUnitDescriptor FAR *featureDesc - far pointer to feature  */
/*                      unit descriptor                               */
/*                                                                    */
/* EXIT-NORMAL: no of logical channels described in feature unit      */
/*              (this does not include master channel)                */
/*                                                                    */
/* EXIT-ERROR: none                                                   */
/*                                                                    */
/* EFFECTS: none                                                      */
/*                                                                    */
/* INTERNAL REFERENCES:  none                                         */
/*                                                                    */
/* EXTERNAL REFERENCES:  None                                         */
/*                                                                    */
/************************ END OF SPECIFICATIONS ***********************/
VOID SetChannelsCapabilities(USHORT usDeviceIndex, FUnitDescriptor FAR *featureDesc)
{
	register UCHAR ucChannel;
	PADEVICE pDev = &gAudioDevices[usDeviceIndex];

	// clear up channel  nodes, first
	for ( ucChannel=0; ucChannel<=pDev->ucChannels; ucChannel++ )
	{
		pDev->ulDevCh[ucChannel]     = 0L;
		pDev->usMaxVolume[ucChannel] = 0;
		pDev->usMinVolume[ucChannel] = 0;
		pDev->ucMaxBass[ucChannel]   = 0;
		pDev->ucMinBass[ucChannel]   = 0;
		pDev->ucMaxTreble[ucChannel] = 0;
		pDev->ucMinTreble[ucChannel] = 0;
	}
	//  set up channel capabilities for all avalable controls
	for ( ucChannel=0;  ucChannel<=pDev->ucChannels; ucChannel++ )
	{
		pDev->ulDevCh[ucChannel] |= 
		CheckChannelFeature( featureDesc, 
									CHANNEL_FEATURE_MUTE,    
									ucChannel) ? CHANNEL_FEATURE_MUTE : 0;

		pDev->ulDevCh[ucChannel] |= 
		CheckChannelFeature( featureDesc, 
									CHANNEL_FEATURE_VOLUME,  
									ucChannel) ? CHANNEL_FEATURE_VOLUME : 0;

		pDev->ulDevCh[ucChannel] |= 
		CheckChannelFeature( featureDesc, 
									CHANNEL_FEATURE_BASS, 
									ucChannel) ? CHANNEL_FEATURE_BASS : 0;

		pDev->ulDevCh[ucChannel] |= 
		CheckChannelFeature( featureDesc, 
									CHANNEL_FEATURE_MID,     
									ucChannel) ? CHANNEL_FEATURE_MID : 0;

		pDev->ulDevCh[ucChannel] |= 
		CheckChannelFeature( featureDesc, 
									CHANNEL_FEATURE_TREBLE,  
									ucChannel) ? CHANNEL_FEATURE_TREBLE : 0;

		pDev->ulDevCh[ucChannel] |= 
		CheckChannelFeature( featureDesc, 
									CHANNEL_FEATURE_GEQ, 
									ucChannel) ? CHANNEL_FEATURE_GEQ : 0;

		pDev->ulDevCh[ucChannel] |= 
		CheckChannelFeature( featureDesc, 
									CHANNEL_FEATURE_AGAIN, 
									ucChannel) ? CHANNEL_FEATURE_AGAIN : 0;

		pDev->ulDevCh[ucChannel] |= 
		CheckChannelFeature( featureDesc, 
									CHANNEL_FEATURE_DELAY, 
									ucChannel) ? CHANNEL_FEATURE_DELAY : 0;

		pDev->ulDevCh[ucChannel] |= 
		CheckChannelFeature( featureDesc, 
									CHANNEL_FEATURE_BBOOST, 
									ucChannel) ? CHANNEL_FEATURE_BBOOST : 0;

		pDev->ulDevCh[ucChannel] |= 
		CheckChannelFeature( featureDesc, 
									CHANNEL_FEATURE_LOUDNESS,
									ucChannel) ? CHANNEL_FEATURE_LOUDNESS : 0;
		#ifdef DEBUG
		dsPrint4( DBG_DETAILED, 
					 "*****************SetChannelCapabilities : dev=%d ch=%d feature=%xh chans=%d\r\n", 
					 usDeviceIndex,
					 ucChannel, 
					 pDev->ulDevCh[ucChannel],
					 pDev->ucChannels);
		#endif
	}


	return;
}

/********************** START OF SPECIFICATIONS ***********************/
/*                                                                    */
/* SUBROUTINE NAME:  SetControlCapabilities                           */
/*                                                                    */
/* DESCRIPTIVE NAME:  SetControlCapabilities                          */
/*                                                                    */
/* FUNCTION:  The function of this routine is                         */
/*                to Set Control Capabilities  fot the device         */
/*                                                                    */
/* NOTES:                                                             */
/*                                                                    */
/* CONTEXT: Task time                                                 */
/*                                                                    */
/* ENTRY POINT:  SetControlCapabilities                               */
/*    LINKAGE:  CALL NEAR                                             */
/*                                                                    */
/* INPUT:    UCHAR deviceIndex - USB Audio deice internal index       */
/*           PULOMG     pulFlags  - pointer to capability flags       */
/*                                                                    */
/* EXIT-NORMAL: ALWAYS                                                */
/*                                                                    */
/* EXIT-ERROR: none                                                   */
/*                                                                    */
/* EFFECTS: none                                                      */
/*                                                                    */
/* INTERNAL REFERENCES:  none                                         */
/*                                                                    */
/* EXTERNAL REFERENCES:  None                                         */
/*                                                                    */
/************************ END OF SPECIFICATIONS ***********************/

VOID SetControlCapabilities(UCHAR deviceIndex, PULONG pulFlags)
{
	register UCHAR ucChannel;

	if(deviceIndex!=INDEX_NOT_AVAILABLE)   // 04/50/2000 MB
   //set up volume treble and bass capabilities
	for ( ucChannel=0; ucChannel<=gAudioDevices[deviceIndex].ucChannels; ucChannel++ )
	{
		*pulFlags|=(gAudioDevices[deviceIndex].ulDevCh[ucChannel]&CHANNEL_FEATURE_VOLUME)?VOLUME:0;
		*pulFlags|=(gAudioDevices[deviceIndex].ulDevCh[ucChannel]&CHANNEL_FEATURE_TREBLE)?TREBLE:0;
		*pulFlags|=(gAudioDevices[deviceIndex].ulDevCh[ucChannel]&CHANNEL_FEATURE_BASS)?BASS:0;
	}


	return; 
}


/********************** START OF SPECIFICATIONS ***********************/
/*                                                                    */
/* SUBROUTINE NAME:  InitNewDevice                                    */
/*                                                                    */
/* DESCRIPTIVE NAME:  Get Channel Count                               */
/*                                                                    */
/* FUNCTION:  The function of this routine is to initialize new       */
/*        attached USB device for  active stream  playback            */
/*                                                                    */
/* NOTES:                                                             */
/*                                                                    */
/* CONTEXT: Task time                                                 */
/*                                                                    */
/* ENTRY POINT:  InitNewDevice                                        */
/*    LINKAGE:  CALL NEAR                                             */
/*                                                                    */
/* INPUT:  None                                                       */
/*                                                                    */
/* EXIT-NORMAL: NO_ERROR                                              */
/*                                                                    */
/* EXIT-ERROR: ERROR                                                  */
/*                                                                    */
/* EFFECTS: none                                                      */
/*                                                                    */
/* INTERNAL REFERENCES:  none                                         */
/*                                                                    */
/* EXTERNAL REFERENCES:  None                                         */
/*                                                                    */
/************************ END OF SPECIFICATIONS ***********************/

RC InitNewDevice( UCHAR  ucActiveDeviceIndex, PSTREAM pStream )   // 04/05/2000 MB
{
	ULONG  ulDataType       =  (ULONG)pStream->usMode;	// active stream datatype
	ULONG  ulSamplingRate   =  pStream->ulSamplingFreq; // active stream sampling frequence                                               
	ULONG  ulChannels       =  pStream->usChannels;	// active stream channels setting
	ULONG  ulBitsPerSample  =  pStream->ulBitsPerSample;	// active stream bits per sample value
	PADEVICE  paDevice      =  &gAudioDevices[ucActiveDeviceIndex];


	// to prevent errors
	if ( !pStream || !paDevice->pDeviceInfo)  // 08/01/2000 MB - to check for detached devices
		return (ERROR);

	#ifdef DEBUG
	dsPrint3(DBG_DETAILED, "InitNewDevice : ulOp=%ld OutPipe=%d InPipe=%d\r\n", 
				pStream->ulOperation, 
				paDevice->ucWritePipe, 
				paDevice->ucReadPipe );
		#endif

	if ( (pStream->ulOperation==OPERATION_PLAY && !paDevice->ucWritePipe) ||
		  (pStream->ulOperation==OPERATION_RECORD && !paDevice->ucReadPipe)||
		  (pStream->ulOperation!=OPERATION_PLAY  && pStream->ulOperation!=OPERATION_RECORD)
		)
	{
		return (ERROR);
	}

	// first, check for new attached USB audio device capabilities

	if ( ! GetBestFit((PUCHAR)&(paDevice->pDeviceInfo->configurationData), // device specific information
							paDevice->bNumConfigurations,					// no of the configurations for the device
							paDevice->devConf->bConfigurationValue, 
							paDevice->ucStreamingInterface,				// USB streaming interaface value for the device
							(PULONG)&ulDataType,								// data type
							(PULONG)&ulSamplingRate,						// sampling frequence
							(PULONG)&ulChannels,								// channels No
							(PULONG)&ulBitsPerSample,						// bits per sample
							(PUCHAR)&paDevice->ucAltInterface) )		// alternative interface
	{
		#ifdef DEBUG
		dsPrint( DBG_CRITICAL, 
					"USBAUD : InitNewDevice : GetBestFit error!\r\n");
		#endif
		return (ERROR);
	}


	// check for correct stream data type
	if ( (ulDataType!=pStream->usMode) && (ulDataType>2) )
	{
		#ifdef DEBUG
		dsPrint2( DBG_CRITICAL, 
					 "USBAUD : InitNewDevice error invalid datatype=%xh expected=%lxh\r\n", 
					 GlobalTable.paStream->usMode, 
					 ulDataType);
		#endif
		return (ERROR);
	}

	// if device cannot support specified stream data setting
	// it allows for sampling rate to be in range +-10%
	if ( (ulSamplingRate   >  pStream->ulSamplingFreq+pStream->ulSamplingFreq/10)||
		  (ulSamplingRate   <  pStream->ulSamplingFreq-pStream->ulSamplingFreq/10)||
		  (ulChannels       !=  pStream->usChannels)||      
		  (ulBitsPerSample  !=  pStream->ulBitsPerSample) )
	{
		#ifdef DEBUG
		dsPrint3( DBG_CRITICAL, 
					 "USBAUD : InitNewDevice : Error : Invalid format -  freq=%ld bits=%ld chan=%d\r\n",
					 pStream->ulSamplingFreq,
					 pStream->ulBitsPerSample,
					 pStream->usChannels);

		dsPrint4( DBG_CRITICAL, 
					 "USBAUD : InitNewDevice : expected data format = freq=[%ld..%ld] bits=%ld chan=%d\r\n",
					 ulSamplingRate-ulSamplingRate/10,
					 ulSamplingRate+ulSamplingRate/10,
					 ulBitsPerSample,
					 ulChannels
				  );
		#endif

		return (ERROR);
	}

	{
		register UCHAR ucChannel;

		// all new device channels from 0 (master channel) settings must be updated to match previous ones
		for ( ucChannel=0; ucChannel<=paDevice->ucChannels;  ucChannel++ )
		{
			// if channel supports VOLUME control setting
			if ( paDevice->ulDevCh[ucChannel]&CHANNEL_FEATURE_VOLUME )
			{
				#ifdef DEBUG
				dsPrint4( DBG_SPECIFIC, 
							 "USBAUD: Set USB chan #%d Volume to = %xh min=%xh max=%xh\r\n", 
							 ucChannel, 
							 paDevice->usLeftVolume, 
							 paDevice->usMinVolume[ucChannel],
							 paDevice->usMaxVolume[ucChannel]);
				#endif
				// set up volume
				SetUnitControl( ucActiveDeviceIndex, 
									 VOLUME_CONTROL, 
									 ucChannel, 
									 sizeof(paDevice->usLeftVolume), 
									 (PUCHAR)&paDevice->usLeftVolume);
			}
			// if channel supports TREBLE control setting
			if ( paDevice->ulDevCh[ucChannel]&CHANNEL_FEATURE_TREBLE )
			{
				#ifdef DEBUG
				dsPrint4( DBG_SPECIFIC, 
							 "USBAUD: Set USB chan #%d Treble to = %xh min=%xh max=%xh\r\n",
							 ucChannel, 
							 paDevice->ucLeftTreble, 
							 paDevice->ucMinTreble[ucChannel],
							 paDevice->ucMaxTreble[ucChannel]);
				#endif
				// set up treble
				SetUnitControl( ucActiveDeviceIndex, 
									 TREBLE_CONTROL, 
									 ucChannel, 
									 sizeof(paDevice->ucLeftTreble), 
									 (PUCHAR)&paDevice->ucLeftTreble);
			}

			// if channel support BASS control setting
			if ( paDevice->ulDevCh[ucChannel]&CHANNEL_FEATURE_BASS )
			{
				#ifdef DEBUG
				dsPrint4(DBG_SPECIFIC, 
							"USBAUD: Set USB chan #%d Bass to = %xh min=%xh max=%xh\r\n", 
							ucChannel, 
							paDevice->ucLeftBass,
							paDevice->ucMinBass[ucChannel],
							paDevice->ucMaxBass[ucChannel]);
				#endif
				// set up bass
				SetUnitControl( ucActiveDeviceIndex, 
									 BASS_CONTROL, 
									 ucChannel, 
									 sizeof(paDevice->ucLeftBass), 
									 (PUCHAR)&paDevice->ucLeftBass);
			}
		}
	}
	#ifdef DEBUG
	dsPrint(DBG_DETAILED, "InitNewDevice OK\r\n");
	#endif
	return (NO_ERROR);
}
/********************** START OF SPECIFICATIONS ***********************/
/*                                                                    */
/* SUBROUTINE NAME:  SetStreamState                                   */
/*                                                                    */
/* DESCRIPTIVE NAME:  SetStreamState                                  */
/*                                                                    */
/* FUNCTION:    The function of this routine is to set                */
/*                specified stream state                              */
/*                                                                    */
/* NOTES: This function is not required for MMPM/2                    */
/*                                                                    */
/* CONTEXT: Task time                                                 */
/*                                                                    */
/* ENTRY POINT:  GetChannelCount                                      */
/*    LINKAGE:  CALL NEAR                                             */
/*                                                                    */
/* INPUT: PSTREAM pStream - pointer to the stream                     */
/*        USHORT usState - the state of the stream                    */
/*                                                                    */
/*                                                                    */
/* EXIT-NORMAL: always                                                */
/*                                                                    */
/* EXIT-ERROR: N/A                                                    */
/*                                                                    */
/* EFFECTS: none                                                      */
/*                                                                    */
/* INTERNAL REFERENCES:  none                                         */
/*                                                                    */
/* EXTERNAL REFERENCES:  None                                         */
/*                                                                    */
/************************ END OF SPECIFICATIONS ***********************/


VOID SetStreamState(PSTREAM pStream, USHORT usState)
{
	//check for the correct stream first
	if ( !pStream )
		return;
// this function is not requried by MMPM/2 
	switch ( usState )
	{
		case STREAM_STATE_STOP:
			break;
		case STREAM_STATE_START:
			break;
		case STREAM_STATE_PAUSE:
			break;
		case STREAM_STATE_RESUME:
			break;
		default:
			break;
	}
	return;
}
/********************** START OF SPECIFICATIONS ***********************/
/*                                                                    */
/* SUBROUTINE NAME:  GetUnitControlValue                              */
/*                                                                    */
/* DESCRIPTIVE NAME:  Get Unit Control Value                          */
/*                                                                    */
/* FUNCTION:  The function of this routine is to calculate USB        */
/*                related setting control value number of             */
/*                                                                    */
/* CONTEXT: Task time                                                 */
/*                                                                    */
/* ENTRY POINT:  GetUnitControlValue                                  */
/*    LINKAGE:  CALL NEAR                                             */
/*                                                                    */
/* INPUT:  USHORT usValue  - OS/2 standard control value              */
/*      USHORT usMinValue  - minimal USB control value for the device */
/*      USHORT usMaxValue  - maximum USB control value for the device */
/*                                                                    */
/*                                                                    */
/* EXIT-NORMAL: Calculated value of the control setting for USB device*/
/*                                                                    */
/* EXIT-ERROR: none                                                   */
/*                                                                    */
/* EFFECTS: none                                                      */
/*                                                                    */
/* INTERNAL REFERENCES:  none                                         */
/*                                                                    */
/* EXTERNAL REFERENCES:  None                                         */
/*                                                                    */
/************************ END OF SPECIFICATIONS ***********************/
SHORT GetUSBLevel ( SHORT pmValue, SHORT usbMin, SHORT usbMax);
USHORT GetUnitControlValue(USHORT usValue, USHORT usMinValue, USHORT usMaxValue )
{
   USHORT usCurValue=(USHORT)GetUSBLevel ( (SHORT)usValue, (SHORT)usMinValue, (SHORT)usMaxValue);
#ifdef DEBUG
   	dsPrint4(DBG_DETAILED, "GetUnitControlValue : val %x, min %x, max %x, usb %x\r\n",
               usValue, usMinValue, usMaxValue, usCurValue);
#endif
   return ( usCurValue ); // calculate and return device control setting value
}
/********************** START OF SPECIFICATIONS ***********************/
/*                                                                    */
/* SUBROUTINE NAME:  GetNextBuffer                                    */
/*                                                                    */
/* DESCRIPTIVE NAME:  Get Next Buffer                                 */
/*                                                                    */
/* FUNCTION:  The function of this routine is to obtain pointer       */
/*                to next buffer.                                     */
/*                                                                    */
/* CONTEXT: Task time                                                 */
/*                                                                    */
/* ENTRY POINT:  GetNextBuffer                                        */
/*    LINKAGE:  CALL NEAR                                             */
/*                                                                    */
/* INPUT:  PSTREAM pStream - a pointer to a stream                    */
/*                                                                    */
/* EXIT-NORMAL: STATUS_DONE                                           */
/*                                                                    */
/* EXIT-ERROR: STATUS_DONE|STERR  - next buffer does not exist        */
/*                                                                    */
/* EFFECTS: none                                                      */
/*                                                                    */
/* INTERNAL REFERENCES:  none                                         */
/*                                                                    */
/* EXTERNAL REFERENCES:  None                                         */
/*                                                                    */
/************************ END OF SPECIFICATIONS ***********************/

RC GetNextBuffer (PSTREAM pStream)
{
	register USHORT i;

	if ( !pStream )
	{
		return (STATUS_DONE|STERR);
	}


	// check the next buffer for data
	i  = pStream->usCurrIOBuffIndex;
	pStream->usLastIOBuffIndex = i;

	// check for wrap
	if ( ++i==MAXIOBUFFS )
	{
		i=0;
	}

	if ( !pStream->IOBuff[i].bEmpty )
	{
		pStream->usCurrIOBuffIndex = i;
	}
	else
	{ // the next buffer is empty (already playbacked)
	 #ifdef DEBUG
		dsPrint(DBG_SPECIFIC, 
				  "USBAUD: GetNextBuffer : No more buffers to send\r\n");
	 #endif
		return (STATUS_DONE|STERR);
	}
	// next buffer can be played
	return (STATUS_DONE);
}

/********************** START OF SPECIFICATIONS ***********************/
/*                                                                    */
/* SUBROUTINE NAME:  FindMatchingSampleRate                           */
/*                                                                    */
/* DESCRIPTIVE NAME:  Find Matching Sample Rate                       */
/*                                                                    */
/* FUNCTION:  The function of this routine is to find closest possible*/
/*             sampling rate                                          */  
/* NOTES:                                                             */
/*                                                                    */
/* CONTEXT: Task time                                                 */
/*                                                                    */
/* ENTRY POINT:  FindMatchingSampleRate                               */
/*    LINKAGE:  CALL NEAR                                             */
/*                                                                    */
/* INPUT: PULONG pulSampleRate - pointer to sampling rate value       */
/*                                                                    */
/* EXIT-NORMAL: Sampling rate                                         */
/*                                                                    */
/* EXIT-ERROR: none                                                   */
/*                                                                    */
/* EFFECTS: none                                                      */
/*                                                                    */
/* INTERNAL REFERENCES:  none                                         */
/*                                                                    */
/* EXTERNAL REFERENCES:  None                                         */
/*                                                                    */
/************************ END OF SPECIFICATIONS ***********************/

#define distance(x, y) ((x>y)? x-y : y-x)

USHORT FindMatchingSampleRate(PULONG pulSampleRate)
{
	ULONG  ulMinDistance = 0x7fffffff;
	ULONG  aulSuppSampleRates[] = {11025, 22050, 44100, 8000};	//possible sampling rate setting
	USHORT usMatching=0;
	USHORT us;

	for ( us = 0; us < 4; us++ )
	{
		ULONG ulDistance = distance( *pulSampleRate, aulSuppSampleRates[us] );
		if ( ulDistance < ulMinDistance )
		{
			ulMinDistance = ulDistance;
			usMatching = us;
		}
	}
	*pulSampleRate = aulSuppSampleRates[usMatching];
	return (usMatching);
}


/********************** START OF SPECIFICATIONS ***********************/
/*                                                                    */
/* SUBROUTINE NAME:  GetBuffSize                                      */
/*                                                                    */
/* DESCRIPTIVE NAME:  Get Audio Data Buffer Size                      */
/*                                                                    */
/* FUNCTION:  The function of this routine is to calculate USB related*/
/*            audio buffer size                                       */
/*                                                                    */
/* NOTES:                                                             */
/*                                                                    */
/* CONTEXT: Task time                                                 */
/*                                                                    */
/* ENTRY POINT:  GetBuffSize                                          */
/*    LINKAGE:  CALL NEAR                                             */
/*                                                                    */
/* INPUT:  PSTREAM pStream - pointer to the active stream             */
/*                                                                    */
/* EXIT-NORMAL: ALWAYS                                                */
/*                                                                    */
/* EXIT-ERROR: NEVER                                                  */
/*                                                                    */
/* EFFECTS: NONE                                                      */
/*                                                                    */
/* INTERNAL REFERENCES:  NONE                                         */
/*                                                                    */
/* EXTERNAL REFERENCES:  NONE                                         */
/*                                                                    */
/************************ END OF SPECIFICATIONS ***********************/

ULONG GetBuffSize(PSTREAM pStream)
{
	// if invalid data, then return default value for USB buffer size
	if ( !pStream || !pStream->usFrameSize )
		return (DEFAULT_IOBUFF_SIZE);

	//else calculate value 
	return ((DEFAULT_IOBUFF_SIZE/pStream->usFrameSize+1)*pStream->usFrameSize);
}


/********************* START OF SPECIFICATIONS *********************
* SUBROUTINE NAME: GetStreamEntry
*
* DESCRIPTIVE NAME: Get the stream table entry.
*
* FUNCTION: To search the stream table finding a match with the given parm.
*
* NOTES: This routine is called internally.
*
* ENTRY POINTS:
*
*     LINKAGE:   CALL near
*
* INPUT: pointer to stream table, stream handle to find
*
* EXIT-NORMAL: NO_ERROR
*
* EXIT_ERROR: ERROR_INVALID_STREAM if stream not found in table
*
* INTERNAL REFERENCES: none
*
* EXTERNAL REFERENCES: none
*
*********************** END OF SPECIFICATIONS **********************/
RC GetStreamEntry(PSTREAM FAR *ppStream, HSTREAM hStream)
{
	register USHORT  i = 0;
	PSTREAM  pStream = *ppStream;

	if ( !pStream )
		return (ERROR_INVALID_STREAM);

	while ( pStream->hStream != hStream )
	{				 // find stream entry
		if ( ++i >= GlobalTable.usMaxNumStreams )
			return (ERROR_INVALID_STREAM);
		pStream++;
	};
	*ppStream = pStream;
	return (NO_ERROR);
}






/********************* START OF SPECIFICATIONS *********************
* SUBROUTINE NAME: GetStreamEntry
*
* DESCRIPTIVE NAME:
*
* FUNCTION: 
*
* NOTES: 
*
* ENTRY POINTS:
*
*     LINKAGE:
*
* INPUT: 
*
* EXIT-NORMAL: 
*
* EXIT_ERROR: 
*
* INTERNAL REFERENCES:
*
* EXTERNAL REFERENCES:
*
*********************** END OF SPECIFICATIONS **********************/

PADEVICE  GetActiveDevice( ULONG ulOperation )
{
   UCHAR    ucDeviceIndex=GetActiveDeviceIndex(ulOperation);
   PADEVICE paDevice;

	#ifdef DEBUG
	dsPrint4( DBG_SPECIFIC, 
				 "USBAUD: GetActiveDevice entry. Op=%ld Wr=%d Rd=%d, devIndex=%d\r\n",
				 ulOperation,
				 GlobalTable.bWDevIdx,
				 GlobalTable.bRDevIdx,
             ucDeviceIndex
			  );
	#endif
   
   if(ucDeviceIndex != INDEX_NOT_AVAILABLE)
      paDevice=(PADEVICE)&gAudioDevices[ucDeviceIndex];
   else
      paDevice=NULL;

	#ifdef DEBUG
   if(!paDevice)
	dsPrint(DBG_CRITICAL,
			  "USBAUD: GetActiveDevice ERROR!\r\n");
	#endif

	return (paDevice);
}

/********************* START OF SPECIFICATIONS *********************
* SUBROUTINE NAME: GetActiveDeviceIndex
*
* DESCRIPTIVE NAME: 
*
* FUNCTION: 
*
* NOTES: 
*
* ENTRY POINTS:
*
*     LINKAGE: 
*
* INPUT: 
*
* EXIT-NORMAL: 
*
* EXIT_ERROR: 
*
* INTERNAL REFERENCES: 
*
* EXTERNAL REFERENCES: 
*
*********************** END OF SPECIFICATIONS **********************/
UCHAR     GetActiveDeviceIndex(ULONG ulOperation)
{
   UCHAR    ucDeviceIndex=INDEX_NOT_AVAILABLE;

	#ifdef DEBUG
	dsPrint3( DBG_SPECIFIC, 
				 "USBAUD: GetActiveDeviceIndex entry. Op=%ld Wr=%d Rd=%d\r\n",
				 ulOperation,
				 GlobalTable.bWDevIdx,
				 GlobalTable.bRDevIdx
			  );
	#endif
	if ( ulOperation==OPERATION_PLAY )
	{
		ucDeviceIndex=GlobalTable.bWDevIdx;
	}

	if ( ulOperation==OPERATION_RECORD )
	{
		ucDeviceIndex=GlobalTable.bRDevIdx;
	}

   if(ucDeviceIndex!=INDEX_NOT_AVAILABLE && !gAudioDevices[ucDeviceIndex].pDeviceInfo)
      ucDeviceIndex=INDEX_NOT_AVAILABLE;

	#ifdef DEBUG
   if(ucDeviceIndex==INDEX_NOT_AVAILABLE)
   	dsPrint1( DBG_CRITICAL, 
   				"USBAUD: GetActiveDeviceIndex ERROR! Operation %ld.\r\n", ulOperation);
	#endif

	return (ucDeviceIndex);

}

/********************* START OF SPECIFICATIONS *********************
* SUBROUTINE NAME: GetActiveDeviceIndex
*
* DESCRIPTIVE NAME: 
*
* FUNCTION: 
*
* NOTES: 
*
* ENTRY POINTS:
*
*     LINKAGE: 
*
* INPUT: 
*
* EXIT-NORMAL: 
*
* EXIT_ERROR: 
*
* INTERNAL REFERENCES: 
*
* EXTERNAL REFERENCES: 
*
*********************** END OF SPECIFICATIONS **********************/
VOID     CloseISODevice(UCHAR ucDeviceIndex, ULONG ulOperation)
{
	PSTREAM  pStream=GlobalTable.paStream;
   USHORT   streamIndex;

   for ( streamIndex = 0; streamIndex < GlobalTable.usMaxNumStreams; streamIndex++, pStream++ )
   {
      if ( (pStream->ulSysFileNum == -1) )
         continue;
      if ( pStream->ulOperation != ulOperation )
         continue;

      if(pStream->ucDeviceIndex!=ucDeviceIndex)
         continue;

      ISOClose(pStream);

   } //end for
}


